function [nu,Z0,Xh,Yh,Ag,Bg,Cg] = obj_nu(c,g,lpvsys,ptrajectory,tdata)
% Computing the induced L2 norm of the discrete time LTI system associated
% with a periodic LTV system obtained by evaluating the lpv system lpvsys 
% over the periodic scheduling trajectory defined by (c,ptrajectory,tdata). 


% Set options
options = odeset('RelTol',1e-6,'AbsTol',1e-9);

% State dimension
nx=lpvsys([]);
h=ptrajectory([],c,tdata,-1);


% Integrate Z on the interval [h,0]
Zfh = @(t,Zvec) Zdotfun(t,Zvec,lpvsys,ptrajectory,g,c,tdata);
Zh = zeros(nx);
[Tz,Zvec]=ode45(Zfh,[h,0],Zh(:),options);
if abs(Tz(end))==0 && norm(Zvec(end,:))<inf,
    % Bounded Solution for Z => ||D^||<g
    Z0=reshape(Zvec(end,:),nx,nx);
    Z0=(Z0+Z0')/2;
    
    % Integrate X on the interval [0,h]
    Xfh = @(t,Xvec) Xdotfun(t,Xvec,lpvsys,ptrajectory,g,c,tdata,Tz,Zvec);
    X0 = eye(nx);
    [~,Xvec]=ode45(Xfh, [0 h],X0(:),options);
    Xh=reshape(Xvec(end,:),nx,nx);
    
    % Integrate Y on the interval [0,h]
    Yfh = @(t,Yvec) Ydotfun(t,Yvec,lpvsys,ptrajectory,g,c,tdata);
    Y0 = zeros(nx);
    [~,Yvec]=ode45(Yfh, [0 h],Y0(:),options);
    Yh=reshape(Yvec(end,:),nx,nx);
    Yh=(Yh+Yh')/2;
    
    % Construct lifted discrete-time system
    Ag=Xh;
    %Bg=chol(Yh)';
    %Cg=chol(Z0);
    Bg=sqrtm(Yh)';
    Cg=sqrtm(Z0);

    % Compute norm of discrete-time system
    normtol = 1e-10;
    sysd=ss(Ag,Bg,Cg,0,-1);
    nu=norm(sysd,inf,normtol);
    
    
    
  
   
else
    % Unbounded Solution for Z
    % XXX How to handle this
    %cineq = -inf;
    disp('Warning: Unbounded solution for Z');
    nu=NaN; Q=[]; Yh=[]; Xh=[]; Z0=[]; Ag=[]; Bg=[]; Cg=[];
end








function Zdot=Zdotfun(t,Zvec,lpvsys,ptrajectory,g,c,tdata)

% Construct parameter vector at time t from basis functions
p=ptrajectory(t,c,tdata);

% Evaluate state-space matrices of LPV system at current parameter vector
[A,B,C,D]=lpvsys(p);
[nz,nw] = size(D);
nx = size(A,1);

% Convert Z from vector to matrix form
Z = reshape(Zvec,[nx nx]);

% Compute Zdot
% (Use Eqn 17 in 2009 Automatica paper by Cantoni and Sandberg)
R=g^2*eye(nw)-D'*D;
S=g^2*eye(nz)-D*D';
H22=A+B*(R\D')*C;
Zdot=-H22'*Z-Z*H22-g*Z*B*(R\B')*Z-g*C'*(S\C);

% Vectorize derivative of Z
Zdot = Zdot(:);


function Xdot=Xdotfun(t,Xvec,lpvsys,ptrajectory,g,c,tdata,Tz,Zvec)

% Construct parameter vector at time t from basis functions
p=ptrajectory(t,c,tdata);

% Evaluate state-space matrices of LPV system at current parameter vector
[A,B,C,D]=lpvsys(p);
[~,nw] = size(D);
nx = size(A,1);

% Use linear interpolation to evaluate Z(t) from simulation results
%i = min(find(t>=Tz));
i = find( t>= Tz, 1);
if i>1
    alpha = (t-Tz(i)) / (Tz(i-1)-Tz(i));
    Zvect = (1-alpha)*Zvec(i,:)+alpha*Zvec(i-1,:);
else
    Zvect = Zvec(1,:);
end

% Convert Xvec and Zvect from vector to matrix form
Z = reshape(Zvect,[nx nx]);
X = reshape(Xvec,[nx nx]);

% Compute Xdot
% (Use Eqn 18 in 2009 Automatica paper by Cantoni and Sandberg)
R=g^2*eye(nw)-D'*D;
H22=A+B*(R\D')*C;
Xdot=( H22 + g*B*(R\B')*Z )*X;

% Vectorize derivative of X
Xdot = Xdot(:);


function Ydot=Ydotfun(t,Yvec,lpvsys,ptrajectory,g,c,tdata)

% Construct parameter vector at time t from basis functions
p=ptrajectory(t,c,tdata);

% Evaluate state-space matrices of LPV system at current parameter vector
[A,B,C,D]=lpvsys(p);
[nz,nw] = size(D);
nx = size(A,1);

% Convert Yvec from vector to matrix form
Y = reshape(Yvec,[nx nx]);

% Compute Ydot
% (Use Eqn 19 in 2009 Automatica paper by Cantoni and Sandberg)
R=g^2*eye(nw)-D'*D;
S=g^2*eye(nz)-D*D';
H22=A+B*(R\D')*C;
Ydot=(H22*Y+Y*H22'+g*Y*C'*(S\C)*Y+g*B*(R\B'));

% Vectorize derivative of Y
Ydot = Ydot(:);

