function [out1,out2]=traj_sinusoids(t,c,tdata,task,pRange,pRate,hRange)
% Manipulations on a 1-dimensional sinusod scheduling trajectory  

% Concept:
% The scheduling trajectory p(t) is constructed as a linear combination of sinusoidal   
% basis functions. c contains the coefficients and the period h, while tdata defines the
% basis functions. The structure of tdata is the following:
% tdata={fmp,msk}, where fmp is an integer array of frequency multipliers
% and msk is an array with entries 0,1 or 2. In order that the sum of
% sinusoidal functions will be periodic the frequency of each basis
% function should be an integer multiple of the basis frequency. The array fmp contains 
% these integer values. The basis frequency is defined as om=2*pi/h, where the 
% period h is the last component of c, i.e. h=c(end). 
% msk defines the basis functions: if msk(i)==0 then the basis function at
% frequency fmp(i)*om is sinus, if msk(i)==1 then the basis function is
% cosinus and if msk(i)==2 then the basis function is the lin. combination
% of a sinus and a cosinus basis functions. 
%
% Example: fmp=[1 2 4], msk=[0 2 1]
% p(t)=c(1)+c(2)*sin(om*t)+c(3)*sin(2*om*t)+c(4)*cos(2*om*t)+c(5)*cos(4*om*t);
% and length(c)=6, h=c(6), om=2*pi/h;
%
%
% Allowed function calls:
%   h=traj_pwcder([],c,tdata,-1); 
%       gives back the period h
%   [p,pdot]=traj_pwcder(t,c,tdata) or [p,pdot]=traj_pwcder(t,c,tdata,1)
%       evaluates the trajectory at time instants t (t can be an array) and
%       returns p(t(k)) and pdot(t(k)) k=1..length(t)
%   [A,b]=traj_pwcder([],[],tdata,pRange,pRate,hRange)
%       Forms linear inequality and equality constraints for c s.t. if some
%       c=c1 satisfies A*c1<=b then the trajectory defined by
%       (c1,tdata) satisfies the rate and range bounds given in pRange and h bounds
%       defined in hRange, i.e. 
%       pRange(1) <= p(t) <= pRange(2), pRate(1) <= pdot(t) <= pRate(2)
%       hRange(1) <= h <= hRange(2)

if nargin==0 %test
    
    msk=[0 1 2];
    fmp=[1 2 3];
    tdata={fmp,msk};
    c=[1 1 -1 2 3.4 2*pi];
    
    t=0:0.01:4*pi;
    [p,pdot]=traj_sinusoids(t,c,tdata)
    close all;
    subplot(2,1,1); plot(t,p);
    subplot(2,1,2); plot(t,pdot);
    
    pRange=[2 7];
    pRate=[-1.8 1.8];
    hRange=[0.1 5]
    [Alp,Blp]=traj_sinusoids([],[],tdata,1,pRange,pRate,hRange)
    
    return
end

if nargin==3, task=0; end;

if task==-1, 
    h=c(end); 
    out1=h; 
    return; 
elseif task==0
    t=t(:); Nt=numel(t); 
else
    Nt=100; t=linspace(0,1,Nt)';
end;

[fmp,msk]=deal(tdata{:});
nofc=1+length(fmp)+sum(msk>1);

B=ones(Nt,nofc);
Bdot=zeros(Nt,nofc);

if task==0, h=c(end); else h=1; end;
    
k=1; w=2*pi*fmp; a=1/h;
for i=1:length(w)
    if msk(i)==0 || msk(i)==2,
        k=k+1;
        B(:,k)=sin(w(i)*a*t);
        Bdot(:,k)=w(i)*a*cos(w(i)*a*t);
    end;
    if msk(i)==1 || msk(i)==2,
        k=k+1;
        B(:,k)=cos(w(i)*a*t);
        Bdot(:,k)=-w(i)*a*sin(w(i)*a*t);
    end;
end;
    
switch task
    case 0,
        c=c(:);
        p=B*c(1:end-1); pdot=Bdot*c(1:end-1);
        out1=p; out2=pdot;
    case 1,
        if pRate(2)~=-pRate(1), error('Rate bounds must be symmetric to 0.'); end; 
        ratemax=pRate(2);
        B=[B zeros(Nt,1)]; Bdot1=[-Bdot -ratemax*ones(Nt,1)]; Bdot2=[Bdot -ratemax*ones(Nt,1)];
        Alp1=[B;-B;Bdot1;Bdot2]; 
        Blp1=kron([pRange(2); -pRange(1); 0; 0],ones(Nt,1)); 
        
        if hRange(1)>hRange(2) || hRange(1)<0, error('The domain of h must be positive.'); end;
        Alp2=[zeros(1,nofc) 1;zeros(1,nofc) -1]; Blp2=[hRange(2);-hRange(1)];
        
        Alp=[Alp1;Alp2]; Blp=[Blp1;Blp2];
        out1=Alp; out2=Blp;
end;
    
    
    
    
  